#include "Configuration.h"
#include "LED.h"

////////// Sigal-LED-Mapping

// Mapping von Signal-LEDs (deren Adresse in der LED-Kette ist das erste Element) auf DCC-Adressen (zweites Element)
// Wenn von dieser DCC-Adresse die Grundstellung empfangen wird, wird die erstangegebene Farbe ausgeleuchtet, das wird
// in der Regel immer Rot sein,
// anderenfalls die zweitangegebene Farbe. Das ist - je nach dahinterstehendem Signalbegriff - Grün oder Gelb
// Drittes Element: speichert aktuellen Status (Grundstellung oder Minuslage)

// Beispiel erste Zeile: die 4. LED in der Kette (erste LED hat Index 0, also die vierte Index 3!) gehört zu einem
// Signal mit der DCC-Adresse 101. In dessen Grundstellung wird Rot, in dessen Minusstellung Grün ausgeleuchtet

LEDMapping mapping[NumSignalLEDs] = {
  { 3, 101, Z21::Plus, false, RED, GREEN }, // ASig Gl. 1
  { 0, 102, Z21::Plus, false, RED, GREEN }, // ASig Gl. 2
  { 5, 160, Z21::Plus, false, RED, GREEN }, // ESig Lak
  { 6, 165, Z21::Plus, false, RED, GREEN }, // ASig Lak
  { 4, 112, Z21::Plus, false, RED, GREEN }, // ESig Lio
};

////////// Fahrstraßendefinition

// Erstes Element: Starttaster, zweites Element: Zieltaster,
// drittes Element: Name der Fahrstraße (nur für Testzwecke)
// viertes Element: kommagetrennte Liste von Schaltvorgängen. Jedes Element beginnt mit
// der DCC-Adresse und endet mit der notwendigen Stellung: "+" für die Grund- und "-" für die Minusstellung

Route routes[NumRoutes] = {
  { GL1, LAK,  "Ausfahrt aus Gl. 1", "25-,27+,28-,47+,48+,49+,50+,65+,29+,30+,160-,101-" },
  { GL2, LAK,  "Ausfahrt aus Gl. 2", "26+,28+,47+,48+,49+,50+,65+,27-,29+,30+,160-,102-" },
  { LAK, GL1,  "Einfahrt nach Gl. 1", "65-,47+,31+,25+,2-,112-" },
  { LAK, GL2,  "Einfahrt nach Gl. 2", "65-,47+,31-,26-,2+,112-" },
  { GL1, ABST, "Abstellen aus Gl. 1", "25-,27-,29+,30+" },
  { GL2, ABST, "Abstellen aus Gl. 2", "26-,29-,30-" },
  { ABST, GL1, "Bereitstellen auf Gl. 1", "30+,29+,27-,25-,2-" },
  { ABST, GL2, "Bereitstellen auf Gl. 2", "30-,29-,26-,2+" }
};
