#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include "Arduino.h"
#include "Z21.h"

/*
   Konfigurationswerte
*/

// ----------------------------------------------------------------------------------------------------
// Netzwerk
// ----------------------------------------------------------------------------------------------------

// Name des WLANs (SSID) und Passwort
#define SSID "Erdgeschoss24"
#define WPASSWD "FrankUndUte"

// ----------------------------------------------------------------------------------------------------
// Z21/DR5000
// ----------------------------------------------------------------------------------------------------

// Z21/DR5000-Adresse und Port
#define Z21_ADDRESS "192.168.178.200"
#define Z21_PORT 21105

// Lebenszeichen (alle ... Millisekunden Zentrale adressieren, um Verbindung aufrechtzuerhalten
#define Z21_HEARTBEAT 30000

// Verzögerung zwischen dem Aussenden zweier Schaltbefehle (in Millisekunden). Weniger technisch notwendig
// als Anlehnung an reale Stellwerkstechnik
#define SETTING_DELAY 500

// ----------------------------------------------------------------------------------------------------
// Stellpult
// ----------------------------------------------------------------------------------------------------

///// Signal-LEDs

// Helligkeit (1 bis 255)
#define BR 25

// Blinkfrequenz in Millisekunden
#define BLINK_FREQ 250

typedef struct LEDMapping {
  int ledAddr;
  int dccAddr;
  Z21::AccState state;
  boolean blink;
  int r0; int g0; int b0; // RGB-Farbe bei DCC-Status 0
  int r1; int g1; int b1; // RGB-Farbe bei DCC-Status 1
};

// Anzahl der verbauten Signal-LEDs (ohne Status-LEDs!)
#define NumSignalLEDs 5

extern LEDMapping mapping[NumSignalLEDs]; // In Configuration.cpp das eigentliche Mapping vornehmen!

///// Status-LEDs (Index in LED-Kette)

#define STATUS_LED1 9
#define STATUS_LED2 8

// ----------------------------------------------------------------------------------------------------
// Fahrstraßen
// ----------------------------------------------------------------------------------------------------

typedef struct Route {
  int startButtonPin;
  int targetButtonPin;
  String routeName;
  String routeElements;
};

// Anzahl der Taster
#define NumButtons 4

// Anzahl der Fahrstraßen
#define NumRoutes 8

// Zunächst empfiehlt es sich, symbolische Namen für Taster zu vergeben, das macht die spätere
// Fahrstraßendefinition übersichtlicher und fehlerfreier (bei mehrfach verwendeten Tasten

// Wert jeder Definition ist die GPIO-Nummer des Pins, an den der Taster angeschlossen ist
#define GL1  16  // Gleis 1
#define GL2   0  // Gleis 2
#define ABST 19  // Abstellgleis 12
#define LAK   5  // Schattenbahnhof "Lak"

extern Route routes[NumRoutes]; // In Configuration.cpp die eigentliche Fahrstraßendefinition vornehmen!

#endif
