#include <LiquidCrystal_I2C.h>
#include <WiFi.h>
#include <Z21.h>

#include "Configuration.h"

#include "LED.h"
#include "Button.h"
#include "RouteSetting.h"

// LCD
#define LCD1602
#define LCD2004
#ifdef LCD1602
LiquidCrystal_I2C lcd1602(0x26, 16, 2); // Wenn Jumper nich geschlossen: Adresse 0x27. Tabelle siehe https://funduino.de/nr-06-zwei-i%C2%B2c-displays-gleichzeitig
#endif
#ifdef LCD2004
LiquidCrystal_I2C lcd2004(0x27, 20, 4);
#endif

// UDP
WiFiUDP Udp;

// ----------------------------------------------------------------------------------------------------
//

void setup() {

  Serial.begin(115200);
  Serial.println("Franky SP");

  // WLAN
  Serial.printf("Verbinde mit WLAN % s ", SSID);
  WiFi.begin(SSID, WPASSWD);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println(" verbunden.");

  // Z21
  Serial.printf("Verbinde mit Z21/DR5000 %s@%d\n", Z21_ADDRESS, Z21_PORT);
  Z21::setIPAddress(Z21_ADDRESS);
  Udp.begin(Z21_PORT);
  Z21::init();

  Z21::LAN_X_SET_TRACK_POWER(true);

#ifdef LCD1602
  lcd1602.init();
  lcd1602.backlight();
  lcd1602.setCursor(0, 0);
  lcd1602.print("Franky SP 1602");
#endif

#ifdef LCD2004
  lcd2004.init();
  lcd2004.backlight();
  lcd2004.setCursor(0, 0);
  lcd2004.print("Franky SP 2004");
#endif

  // Taster des Stellpults
  Button::begin();

  // LEDs des Stellpults
  LED::begin();

  // Signalzustand abfragen (und LEDs entsprechend setzen)
  RouteSetting::getSignalLEDs();

}

// ----------------------------------------------------------------------------------------------------
//

long lastHeartbeat = 0;
long lastBlink = 0;
long cycle = 0;

void loop() {

  // Lebenszeichen
  if (millis() - lastHeartbeat > Z21_HEARTBEAT) {
    Z21::heartbeat();
    lastHeartbeat = millis();
  }

  // Prüfen, ob Rückmeldungen von Zentrale vorliegen
  Z21::receive();

  // Taster abfragen
  Button::loop();

  if (Button::startButton != NULL && Button::targetButton != NULL) {
    RouteSetting::setRoute(Button::startButton, Button::targetButton);
    Button::reset();
  }

  // Blinken erforderlich?
  if (millis() - lastBlink > BLINK_FREQ) {
    for (int l = 0; l < NumSignalLEDs; l++) {
      if (mapping[l].blink) {
        if (cycle % 2 == 0) {
          LED::unset(l, 0);
        } else {
          LED::set(mapping[l].ledAddr,
                   mapping[l].state == Z21::Plus ? mapping[l].r0 : mapping[l].r1,
                   mapping[l].state == Z21::Plus ? mapping[l].g0 : mapping[l].g1,
                   mapping[l].state == Z21::Plus ? mapping[l].b0 : mapping[l].b1,
                   0);
        }
      }
    }
    lastBlink = millis();
    cycle++;
  }
}
