#include "RouteSetting.h"
#include "Button.h"
#include "LED.h"
#include "Configuration.h"

#include <Z21.h>

// Für leicht merkbarere Tasten im Fahrstraßenarray routes[]: Statt dort die GPIO-Nummern
// anzugeben, sollten die Macros einen merkbaren Tasternamen haben, der Wert ist dann der
// GPIO, an den der Taster angeschlossen ist.
// Dies als String notieren (damit routes[] nur mit Strings initialisiert werden muss)

extern int split(String s, char listSeparator, String elements[]);

// ----------------------------------------------------------------------------------------------------
// Abfragen der Signalzustände, um LEDs entsprechend auszuleuchten
// Ausgewertet wird in z21.cpp

void RouteSetting::getSignalLEDs() {

  for (int i = 0; i < NumSignalLEDs; i++) {
    Z21::LAN_X_GET_TURNOUT_INFO(mapping[i].dccAddr);
  }
}


// ----------------------------------------------------------------------------------------------------
// Fahrstraße stellen, die durch Starttaste an pin1 und Zieltaste an pin2 ausgelöst wurde
// (sofern eine solche konfiguriert ist)

void RouteSetting::setRoute(Button* startButton, Button* targetButton) {

  bool found = false;
  String rE[30]; // Wir haben ja Platz - mehr als 30 Elemente werden es wohl nicht ...

  for (int r = 0; r < NumRoutes; r++) {
    if (routes[r].startButtonPin == startButton -> getPin() && routes[r].targetButtonPin == targetButton -> getPin()) {

      found = true;
      Serial.printf("Stelle %s\n", routes[r].routeName.c_str());
      int n = split(routes[r].routeElements, ',', rE);
      for (int i = 0; i < n; i++) {
        LED::animateRouteSetting(i);
        rE[i].trim();

        // Adresse = erster Teil bis auf letztes Zeichen (+/-). Voraussetzung: ist numerisch!!!
        int addr = (rE[i].substring(0, rE[i].length() - 1)).toInt();
        bool plus = rE[i].endsWith("+");

        // Stellen
        Z21::LAN_X_SET_TURNOUT(addr, plus ? Z21::Plus : Z21::Minus);

        delay(SETTING_DELAY);
      }

      // Startsignal-LED suchen und ausleuchten
      for (int e = 0; e < n; e++) {
        int addr = (rE[e].substring(0, rE[e].length() - 1)).toInt();
        boolean plus = rE[e].endsWith("+");
        for (int s = 0; s < NumSignalLEDs; s++) {
          if (mapping[s].dccAddr == addr) {
            LED::set(mapping[s].ledAddr,
                     plus ? mapping[s].r0 : mapping[s].r1,
                     plus ? mapping[s].g0 : mapping[s].g1,
                     plus ? mapping[s].b0 : mapping[s].b1,
                     0);
            LED::setBlink(s, false);
          }
        }
      }

      break;
    }
  }

  Button::startButton = Button::targetButton = NULL;
  LED::animateStartStopButtonStatus(startButton, targetButton);

  if (!found) Serial.printf("Keine Fahrstraße für Tasten an GPIO %d (Starttaste) und %d (Zieltaste)\n", startButton -> getPin(), targetButton -> getPin());

}
