// Die Z21-Library verwendet diese Callbacks, um asynchron Daten zu liefern

#include <Z21.h>
#include "LED.h"
#include "Configuration.h"

extern int split(String s, char listSeparator, String elements[]);

// ----------------------------------------------------------------------------------------------------
// Trace der Befehle (sofern in X21-Library aktiviert

void trace(byte toZ21, String message, String parameter) {
  Serial.println(String(toZ21 == TO_Z21 ? "-> Z21" : "<- Z21") + "|" + message + "|" + parameter);
}

// ----------------------------------------------------------------------------------------------------
//

void trackPowerStateChanged(Z21::TrackPowerState state) {
  Serial.println(state == Z21::On ? "Gleisspannung ein" : "Gleisspannung aus");
}

// ----------------------------------------------------------------------------------------------------
// Schaltinfos

void accessoryInfo(int addr, Z21::AccState state) {

  // Falls das Signal-LED ist, diese entsprechend ausleuchten
  for (int i = 0; i < NumSignalLEDs; i++) {
    if (mapping[i].dccAddr == addr) {
      mapping[i].state = state;

      LED::set(mapping[i].ledAddr,
               state == Z21::Plus ? mapping[i].r0 : mapping[i].r1,
               state == Z21::Plus ? mapping[i].g0 : mapping[i].g1,
               state == Z21::Plus ? mapping[i].b0 : mapping[i].b1,
               0);
      return;
    }
  }

  // Falls das Weiche ist einer hier verwalteten Fahrstraße, wurde sie manuell gestellt
  // Durch Blinken des Startsignals anzeigen, dass hier ein potenzielles Problem vorliegt

  String rE[30]; // Wir haben ja Platz - mehr als 30 Elemente werden es wohl nicht ...

  for (int r = 0; r < NumRoutes; r++) {

    int n = split(routes[r].routeElements, ',', rE);
    for (int e = 0; e < n; e++) {
      rE[e].trim();

      // Adresse
      int addrInRoute = (rE[e].substring(0, rE[e].length() - 1)).toInt();
      bool plusRequired = rE[e].endsWith("+");

      if (addrInRoute == addr) {
        bool plusReceived = state == Z21::Plus;

        // Wird die Weiche in der Fahrstraße in anderer Lage (!) verwendet?
        if (plusRequired != plusReceived) {

          // Dann relevant, wenn die zugehörige Fahrstraße gestellt ist, d.h. das Startsignal auf Fahrt steht
          // Kriterium: eins der Elemente der Fahrstraße entspricht einem Signal im LED-Mapping

          for (int ee = 0; ee < n; ee++) {

            int addrElem = (rE[ee].substring(0, rE[ee].length() - 1)).toInt();
            for (int s = 0; s < NumSignalLEDs; s++) {
              if (mapping[s].dccAddr == addrElem) {

                // Nur Verletzung, wenn diese Fahrstraße auch eingestellt ist, d.h. das Startsignal auf Fahrt steht
                if (mapping[s].state == Z21::Minus) {
                  Serial.printf("\t\t\t\tBlinken von LED %d mit Adresse %d nötig in Fahrstraße %s\n", mapping[s].ledAddr, mapping[s].dccAddr, routes[r].routeName.c_str());
                  LED::setBlink(s, true);
                }

              }
            }
          }
        }
      }

    }
  }

}

// ----------------------------------------------------------------------------------------------------
// Lokinfos

void locoInfo(int addr, boolean forward, int fst, boolean takenOver, int numSpeedSteps, boolean fct[]) {}
